import * as _ from 'lodash';

const random = () => Math.random().toString(36).slice(-6);
const callbackRender = (code) => {
  if (_.isArray(code)) {
    return `{
      ${code.map(callbackRender).join("\n")}
    }`;
  }
  if (_.isFunction(code)) {
    const string = _.toString(code).trim();
    const start = string.indexOf("{");
    const end = string.lastIndexOf("}");
    const func = string.slice(start + 1, end);
    return func;
  }
  return code;
};
class Render {
  elementType;
  children;
  eventCb;
  methods;
  status;
  attr;
  constructor({
    children,
    attr = {},
    event = {},
    status = {},
    elementType = ""
  }) {
    this.elementType = elementType;
    this.children = this.renderChild(children);
    const [evCb, evFunc] = this.renderEvent(event);
    const [stCb, stFunc] = this.renderStatus(status);
    this.eventCb = evCb.concat(stCb);
    this.methods = evFunc.concat(stFunc);
    this.attr = this.renderAttr(attr);
  }
  renderAttr(list) {
    return Object.entries(list).map((it) => {
      const [key, value] = it;
      if (value == null) {
        return key;
      }
      return `${key}="${value}"`;
    });
  }
  renderEvent(event) {
    return Object.entries(event).reduce(
      ([cb, func], [ev, cbCode]) => {
        const id = random();
        if (_.isArray(cbCode)) {
          cb.push(`(${ev})="${this.elementType || "e"}${id}Callback()"`);
          func.push(
            `async ${this.elementType || "e"}${id}Callback() {
            // * ${ev} event callback
            ${cbCode.map((it) => callbackRender(it)).filter((it) => it).join("\n")}}`
          );
          return [cb, func];
        }
        const { params, callback } = cbCode;
        cb.push(
          `(${ev})="${this.elementType || "e"}${id}Callback(${params.join(
            ","
          )})"`
        );
        func.push(
          `async ${this.elementType || "e"}${id}Callback(${params.join(",")}) {
            // * ${ev} event callback
            ${callback.map((it) => callbackRender(it)).join("\n")}}`
        );
        return [cb, func];
      },
      [[], []]
    );
  }
  renderStatus(status) {
    return Object.entries(status).reduce(
      ([cb, func], [ev, cbCode]) => {
        if (cbCode.length === 0) {
          return [cb, func];
        }
        const id = random();
        cb.push(`[${ev}]="${this.elementType || "e"}${id}Status()"`);
        func.push(
          `${this.elementType || "s"}${id}Status() {
            // * ${ev} status status
            return ${cbCode.map((it) => callbackRender(it)).join("\n")}}`
        );
        return [cb, func];
      },
      [[], []]
    );
  }
  renderChild(children) {
    const childrenList = children?.map((it) => it.render()) || [];
    return {
      template: `${childrenList.map((it) => it.template).join(" ")}`,
      imports: [...childrenList.map((it) => it.imports)],
      data: [...childrenList.map((it) => it.data)],
      init: [...childrenList.map((it) => it.init)],
      components: [...childrenList.map((it) => it.components)],
      methods: [
        ...this.methods || [],
        ...childrenList.map((it) => it.methods)
      ],
      resetFn: [...childrenList.map((it) => it.resetFn)],
      elementType: [...childrenList.map((it) => it.elementType)]
    };
  }
  static toCamel(id) {
    return _.upperFirst(_.camelCase(id));
  }
  static callbackRender(data) {
    return callbackRender(data);
  }
}

class Component extends Render {
  id;
  prefix;
  init;
  imports;
  link;
  constructor({ id = "", prefix = "", link = false, imports, children, init }) {
    super({ children });
    if (!id) {
      console.log("Error: Please enter id in Component");
      return;
    }
    this.id = Render.toCamel(id);
    this.prefix = prefix;
    this.init = init;
    this.imports = imports;
    this.link = link ? [{ name: `${Render.toCamel(id)}`, type: "scss" }] : [];
  }
  render() {
    const componentName = `${this.id}`;
    return {
      id: this.id,
      type: "component",
      template: `${this.children.template}`,
      imports: [
        ...this.children.imports,
        {
          target: ["Component", "OnInit"],
          from: "@angular/core",
          ignore: true
        }
      ],
      link: this.link,
      data: [...this.children.data],
      init: [...this.children.init, this.init],
      methods: [...this.children.methods],
      components: [componentName, ...this.children.components]
    };
  }
}

class Module extends Render {
  id;
  link;
  childrens;
  constructor({ id, children }) {
    super({ children });
    if (!id) {
      console.log("Error: Please enter id in Module");
      return;
    }
    this.id = Render.toCamel(id);
    this.link = children.map((it) => {
      const { type } = it.render();
      return { name: `${Render.toCamel(it.id)}`, type };
    });
    this.childrens = children;
  }
  render() {
    return {
      id: this.id,
      type: "module",
      imports: [
        { target: [{ name: "NgModule", type: "base" }], from: "@angular/core" },
        {
          target: [{ name: "CommonModule", type: "module" }],
          from: "@angular/common"
        },
        ...this.children.imports
      ],
      link: [...this.link],
      children: [...this.childrens],
      components: [...this.children.components]
    };
  }
}

class Service {
  id;
  type;
  constructor({ id }) {
    if (!id) {
      console.log("Error: Please enter id in Service");
      return;
    }
    this.id = id;
  }
  render() {
    return {
      type: "service",
      imports: [
        { target: ["NgModule"], from: "@angular/core", ignore: true },
        { target: ["CommonModule"], from: "@angular/common", ignore: true }
      ]
    };
  }
}

class Element extends Render {
  tag;
  class;
  content;
  constructor({ tag, children, content, class: cls }) {
    super({ children });
    this.tag = tag;
    this.class = cls;
    this.content = content;
  }
  render() {
    return {
      type: "element",
      imports: [...this.children.imports],
      template: `<${this.tag} [ngClass]="${this.class}">${this.content}</${this.tag}>`,
      data: [...this.children.data],
      init: [...this.children.init],
      methods: [...this.children.methods]
    };
  }
}

var index = {};

export { Component, Element, Module, Render, Service, index as default };
//# sourceMappingURL=render.js.map
